import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import controlP5.*; 
import processing.serial.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class BeaconInterface extends PApplet {


                  // Import the CP5 library
          // Import the serial library

Serial myPort;                       // Create the serial port object
ControlP5 cp5;                       // Create the CP5 object

// Creation of the object effectCircle
robotCircle myRobotCircle = new robotCircle(500, 4000.0f);
robotCircle robot1Circle = new robotCircle(200, 3400.0f);
robotCircle robot2Circle = new robotCircle(200, 1000.0f);

// Creation of the variable for the GUI
// Console
Textarea consoleArea;
Println console;
// Creation of the lists
ScrollableList serialList;
// Creation of the buttons
Button serialBegin;
Button serialSend;
Button serialStop;
Button serialRefresh;
// State of the buttons
boolean clicSerialBegin = false ;
boolean clicSerialStop = false ;
boolean clicSerialSend = false ;
boolean clicSerialRefresh = false ;
// Creation of the sliders
Slider speed;
// Creation of the toggles
Toggle toggleCoordinate;
// State of the toggles
boolean toggleValue = false;
// GUI Visibility
boolean GUIVisible = true;
// State new message
boolean newPosition = false;


// Communication variables
byte[] receivedBuf = new byte[6];
int[] transformBuf = new int[4];

int distance, angle;


public void setup()
{
  
  frameRate(24);
  noStroke();
  fill(0);

  createGUI();
}

public void draw()
{
  background(255);
  robotDisplay();
  mouseDisplay();

  //myRobotCircle.updateXY(width/2,height/2,300);
  if(newPosition)robot2Circle.updateDA(distance,angle);
}

public void serialEvent(Serial p)
{
  p.readBytesUntil(10,receivedBuf);
  p.clear();
  if(receivedBuf[0]=='b')
  {
    print("Raw datas : ");
    for(int i=0;i<=5;i++)
    {
      print(receivedBuf[i] + " ");
    }
    println();

    for(int i =0;i<=3;i++)
       {
         transformBuf[i] = unsignedByte(receivedBuf[i+1]);
       }
    distance = transformBuf[0] << 8 | transformBuf[1];
    angle = transformBuf[2] << 8 | transformBuf[3];
    print(distance);
    print(" ; ");
    println(angle);
    newPosition = true;
  }
  else
  {
    println("Serial corruption - informations ignored");
    newPosition = false;
  }
}

// Convert a signed Byte ( 127 to - 128 ) to an Int between 0 and 255
public int unsignedByte(byte value)
{
  if(value < 0) return value + 256 ;
  else return value;
}
public void mouseDisplay()
{
  if (toggleValue)showCoordinates(mouseX, mouseY);
}

public void showCoordinates(int pX, int pY)
{
  //create two vector to calculate the angle between them
  PVector v1 = new PVector(1, 0);    // Vector for the origin
  PVector v2 = new PVector(pX-(displayWidth/2), pY-(displayHeight/2));
  fill(0, 102, 153);
  textSize(11);
  text(PApplet.parseInt(dist(displayWidth/2, displayHeight/2, pX, pY))+" cm", pX+7, pY-5);       // the distance
  text(PApplet.parseInt(degrees(PVector.angleBetween(v1, v2)))+" °", pX+7, pY-16);       // the angle
  circle(pX, pY, 5);
}

public void robotDisplay()
{
  // Print the help text at the top right of the interface
  fill(157, 157, 157);
  textSize(11);
  text("Press 'V' to show/hide the GUI",displayWidth-180, 20);
  //----------------------------------------------------
   // Create the robot at the center of the interface
   fill(222, 158, 29);
   circle(displayWidth/2, displayHeight/2, 30);
}

public void createGUI()
{
  //----------------------------------------------------
  // Create the GUI ControlP5 instance
  cp5 = new ControlP5(this);
  // List of the serial port
  serialList = cp5.addScrollableList("dropdown")
     .setCaptionLabel("Liste des liaisons serie")
     .setPosition(20, 20)
     .setSize(180, 160)
     .setBarHeight(20)
     .setItemHeight(20)
     .addItems(Serial.list())
     .setOpen(false)
     ;
  // create a new button with name 'serialBegin'
  serialRefresh = cp5.addButton("serialRefresh")
     .setCaptionLabel("R")
     .setPosition(210,20)
     .setSize(20,20)
     ;
  // create a new button with name 'serialBegin'
  serialBegin = cp5.addButton("serialBegin")
     .setCaptionLabel("Connection")
     .setPosition(240,20)
     .setSize(80,20)
     ;
  // create a new button with name 'serialSend'
  serialSend = cp5.addButton("serialSend")
     .setCaptionLabel("Send parameters")
     .setPosition(20,100)
     .setSize(180,20)
     ;
  // create a new button with name 'serialStop'
  serialStop = cp5.addButton("serialStop")
    .setCaptionLabel("Stop")
    .setPosition(330,20)
    .setSize(40,20)
    ;
  //Add a slider
  speed = cp5.addSlider("speed")
    .setCaptionLabel("Speed")
    .setColorCaptionLabel(color(20,20,20))
    .setPosition(20, 50)
    .setSize(180, 20)
    .setRange(0, 255)
    .setValue(0)
    ;
  // Creation of the console
  consoleArea = cp5.addTextarea("Console")
    .setPosition(20, height-200)
    .setSize(width-40,140 )
    .setFont(createFont("", 10))
    .setLineHeight(14)
    .setColor(color(255))
    .setColorBackground(color(0,44,91,240))
    ;
  // Creation of a toggle for the Visibility of the coordinates
  toggleCoordinate = cp5.addToggle("toggleCoordinate")
    .setCaptionLabel("Show coordinates")
    .setColorCaptionLabel(color(20,20,20))
    .setPosition(20,150)
    .setSize(50,20)
    .setValue(false)
    .setMode(ControlP5.SWITCH)
    ;

   console = cp5.addConsole(consoleArea);

   println("Welcome to the Beacon interface");
   println("press'h' for more help");
   printArray(Serial.list());

   //----------------------------------------------------
   // Bring the dropdown list for serial to the front
   serialList.bringToFront();
   //----------------------------------------------------
}

public void toggleCoordinate(boolean flag)
{
  toggleValue = flag ;
}

public void serialSend()
{
 int speedValue = PApplet.parseInt(speed.getValue());
 myPort.write(PApplet.parseByte(speedValue));
 //myPort.write('\n');
 println("Parameters sent : speed set to " + speedValue);
}

public void serialBegin()
{
 if (clicSerialBegin == false)
 {
   clicSerialBegin = true ;
   serialBegin.setCaptionLabel("re-connect");
 }
 myPort = new Serial(this, Serial.list()[PApplet.parseInt(serialList.getValue())], 9600);
 myPort.buffer(6);
 myPort.bufferUntil('\n');
 println("Serial connection has begin on port " + PApplet.parseInt(serialList.getValue()));
}

public void serialRefresh()
{
  clicSerialRefresh = true ;
  serialList.clear();
  serialList.addItems(Serial.list());
  println("Refreshing the serial list");
}

public void serialStop()
{
  clicSerialStop = true ;
  myPort.stop();
  newPosition = false;
  println("Serial connection has been stopped");
}

public void keyPressed()
{
  if ( key == 'v' ) GUIVisible = !GUIVisible;
  if ( key == 'h' )
  {
    println("Help for the beacon interface");
    println("'h' - Help");
    println("'v' - Visibility of the GUI");
  }

  cp5.get(Textarea.class, "Console").setVisible(GUIVisible);
  cp5.get(Button.class, "serialBegin").setVisible(GUIVisible);
  cp5.get(Button.class, "serialSend").setVisible(GUIVisible);
  cp5.get(Button.class, "serialStop").setVisible(GUIVisible);
  cp5.get(Button.class, "serialRefresh").setVisible(GUIVisible);
  cp5.get(ScrollableList.class, "dropdown").setVisible(GUIVisible);
  cp5.get(Slider.class,"speed").setVisible(GUIVisible);
  cp5.get(Toggle.class,"toggleCoordinate").setVisible(GUIVisible);
}

class robotCircle
{
  final int radiusMax ;     // Max radius of the "searching circle"
  int radius ;              // Radius of the "searching circle"
  int searchTime = 0;
  int searchTimeLast = 0 ;
  float boucle ;            // Duration of a loop in milliseconds by default

  robotCircle(int setRadiusMax, float setBoucle)
  {
    radiusMax = setRadiusMax ;
    radius = radiusMax ;
    boucle = setBoucle ;
    searchTimeLast = millis();
  }

  public void updateXY(int xCircle, int yCircle ,int distance)
  {
    searchTime = millis()- searchTimeLast; // Take the elapsed time
    float nbr = map(searchTime,boucle,0,radiusMax,0);

    radius=radius-PApplet.parseInt(nbr);
    if(radius<=0) radius = radiusMax;
    searchTimeLast = millis();

    float transparency = map(radius,radiusMax,radiusMax/2,40,0);
    float red = map(distance,0,300,255,0);
    float green = 44 + map(distance,0,300,0,255);

    pushMatrix();
    // Transparency evolution
    fill(red,green,91,transparency);
    noStroke();
    ellipse(xCircle, yCircle, radius, radius);

    fill(255,0);
    strokeWeight(abs(map(radius,radiusMax,radiusMax/2,8,0)));
    // Transparency evolution
    stroke(red,green,91,transparency);
    float radiusExtern = map(radius,radiusMax,0,radiusMax,radiusMax+100);
    ellipse(xCircle, yCircle, radiusExtern, radiusExtern);

    // Create the robot circle
    fill(red, green, 0, 125);
    circle(xCircle, yCircle, map(distance,0,300,70,20));

    // Create and show the coordinates if visibility checked
    if (toggleValue) showCoordinates(xCircle, yCircle);

    popMatrix();
  }

  public void updateDA(int distanceCircle, int angleCircle)
  {
    float px = width/2 + cos(radians(angleCircle))*distanceCircle;
    float py = height/2 + sin(radians(angleCircle))*distanceCircle;
    updateXY((int)px, (int)py, distanceCircle);
  }
}
  public void settings() {  size(displayWidth,displayHeight); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "BeaconInterface" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
